@@ -10,8 +10,7 @@ import numpy as np
 import xobjects as xo
 from .general import _print
 
-
-import xpart as xp # To get the right Particles class depending on pyheatail interface state
+import xtrack as xt
 
 logger = logging.getLogger(__name__)
 
@@ -43,7 +42,7 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
                       particle_ref=None,
                       num_particles=None,
                       x=None, px=None, y=None, py=None,
-                      zeta=None, delta=None, pzeta=None,
+                      zeta=None, delta=None, pzeta=None, ptau=None,
                       x_norm=None, px_norm=None, y_norm=None, py_norm=None,
                       zeta_norm=None, pzeta_norm=None,
                       tracker=None,
@@ -54,10 +53,11 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
                       R_matrix=None,
                       W_matrix=None,
                       method=None,
-                      nemitt_x=None, nemitt_y=None,
+                      nemitt_x=None, nemitt_y=None,nemitt_zeta = None,
                       scale_with_transverse_norm_emitt=None,
                       weight=None,
-                      particles_class=None,
+                      s_tol=1e-6,
+                      include_collective=False,
                       **kwargs, # They are passed to the twiss
                     ):
 
@@ -80,13 +80,10 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
             "please call Line.build_tracker() first.")
 
     assert mode in [None, 'set', 'shift', 'normalized_transverse']
-    Particles = xp.Particles # To get the right Particles class depending on pyheatail interface state
+    Particles = xt.Particles  # To get the right Particles class depending on pyheatail interface state
 
     assert 'at_s' not in kwargs, "at_s is not a valid argument for this function"
 
-    if particles_class is not None:
-        raise NotImplementedError
-
     # Deprecation warning for scale_with_transverse_norm_emitt
     if scale_with_transverse_norm_emitt is not None:
         _print('Warning! '
@@ -120,6 +117,7 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
     zeta = (zeta.get() if hasattr(zeta, "get") else zeta)
     delta = (delta.get() if hasattr(delta, "get") else delta)
     pzeta = (pzeta.get() if hasattr(pzeta, "get") else pzeta)
+    ptau = (ptau.get() if hasattr(ptau, "get") else ptau)
     x_norm = (x_norm.get() if hasattr(x_norm, "get") else x_norm)
     px_norm = (px_norm.get() if hasattr(px_norm, "get") else px_norm)
     y_norm = (y_norm.get() if hasattr(y_norm, "get") else y_norm)
@@ -127,13 +125,14 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
     zeta_norm = (zeta_norm.get() if hasattr(zeta_norm, "get") else zeta_norm)
     pzeta_norm = (pzeta_norm.get() if hasattr(pzeta_norm, "get") else pzeta_norm)
 
-    if line is not None and line.iscollective:
+    if line is not None and line.iscollective and not include_collective:
         logger.warning('Ignoring collective elements in particles generation.')
         line = line._get_non_collective_line()
 
-    # Compute ptau from delta
+    # Compute pzeta from delta
     if delta is not None:
         assert pzeta is None
+        assert ptau is None
         if not np.isscalar(delta):
             delta = np.array(delta)
         beta0 = particle_ref._xobject.beta0[0]
@@ -142,6 +141,14 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
                             + 2. * delta_beta0 * beta0 + 1.)**0.5 - 1.
         pzeta = ptau_beta0 / beta0 / beta0
 
+    # Compute pzeta from ptau
+    if ptau is not None:
+        assert pzeta is None
+        assert delta is None
+        if not np.isscalar(ptau):
+            ptau = np.array(ptau)
+        pzeta = ptau / particle_ref._xobject.beta0[0]
+
     if (x_norm is not None or px_norm is not None
             or y_norm is not None or py_norm is not None
             or zeta_norm is not None or pzeta_norm is not None):
@@ -158,9 +165,12 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
     ref_dict = {
         'q0': particle_ref.q0,
         'mass0': particle_ref.mass0,
+        't_sim': particle_ref.t_sim,
         'p0c': particle_ref.p0c[0],
         'gamma0': particle_ref.gamma0[0],
         'beta0': particle_ref.beta0[0],
+        'pdg_id': particle_ref.pdg_id[0],
+        'anomalous_magnetic_moment': particle_ref.anomalous_magnetic_moment[0],
     }
     part_dict = ref_dict.copy()
 
@@ -169,35 +179,40 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
         assert line is not None
         assert mode == 'normalized_transverse'
         if isinstance(at_element, str):
-            at_element = line.element_names.index(at_element)
+            at_element = line._element_names_unique.index(at_element)
         assert R_matrix is None # Not clear if it is at the element or at start machine
         if particle_on_co is not None:
             assert particle_on_co._xobject.at_element == 0
 
     if match_at_s is not None:
-        import xtrack as xt
         assert at_element is not None, (
             'If `match_at_s` is provided, `at_element` needs to be provided and'
             'needs to correspond to the corresponding element in the sequence'
         )
-        # Match at a position where there is no marker and backtrack to the previous marker
-        expected_at_element = np.where(np.array(
-                    line.get_s_elements())<=match_at_s)[0][-1]
-        assert at_element == expected_at_element or (
-                at_element < expected_at_element and
-                      all([ xt._is_aperture(line.element_dict[nn])
-                           or xt._behaves_like_drift(line.element_dict[nn])
-                for nn in line.element_names[at_element:expected_at_element]])), (
-            "`match_at_s` can only be placed in the drifts downstream of the "
-            "specified `at_element`. No active element can be present in between."
-            )
-        (tracker_rmat, _
-            ) = xt.twiss._build_auxiliary_tracker_with_extra_markers(
-                tracker=line.tracker, at_s=[match_at_s],
-                marker_prefix='xpart_rmat_')
-        at_element_line_rmat = tracker_rmat.line.element_names.index(
-                                                                 'xpart_rmat_0')
-        line_rmat = tracker_rmat.line
+        s_elements = line.get_s_elements()
+        s_at_element = s_elements[at_element]
+        if np.abs(match_at_s - s_at_element) < s_tol:
+            at_element_line_rmat = at_element
+            line_rmat = line
+            match_at_s = None
+        else:
+            # Match at a position where there is no marker and backtrack to the previous marker
+            expected_at_element = np.where(np.array(s_elements)<=match_at_s)[0][-1]
+            assert at_element == expected_at_element or (
+                    at_element < expected_at_element and
+                        all([xt._is_aperture(line.element_dict[nn], line)
+                            or xt._behaves_like_drift(line.element_dict[nn], line)
+                    for nn in line._element_names_unique[at_element:expected_at_element]])), (
+                "`match_at_s` can only be placed in the drifts downstream of the "
+                "specified `at_element`. No active element can be present in between."
+                )
+            (tracker_rmat, _
+                ) = xt.twiss._build_auxiliary_tracker_with_extra_markers(
+                    tracker=line.tracker, at_s=[match_at_s],
+                    marker_prefix='xpart_rmat_')
+            at_element_line_rmat = tracker_rmat.line._element_names_unique.index(
+                                                                    'xpart_rmat_0')
+            line_rmat = tracker_rmat.line
     else:
         line_rmat = line
         at_element_line_rmat = at_element
@@ -213,13 +228,15 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
             tw_state = tw.get_twiss_init(at_element=
                 (at_element_line_rmat if at_element_line_rmat is not None else 0))
 
+            # This is not initialized by get_twiss_init
+            tw_state.particle_on_co.at_element = line_rmat._element_names_unique.index(
+                                                        tw_state.element_name)
+
             WW = tw_state.W_matrix
             particle_on_co = tw_state.particle_on_co
         elif W_matrix is None and R_matrix is not None:
             import xtrack.linear_normal_form as lnf
-            WW, _, _ = lnf.compute_linear_normal_form(
-                                R_matrix,
-                                **kwargs)
+            WW, _, _, _ = lnf.compute_linear_normal_form(R_matrix, **kwargs)
         else:
             WW = W_matrix
 
@@ -250,7 +267,11 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
             gemitt_y = (nemitt_y / particle_ref._xobject.beta0[0]
                         / particle_ref._xobject.gamma0[0])
 
-        gemitt_zeta = 1
+        if nemitt_zeta is None:
+            gemitt_zeta = 1
+        else:
+            gemitt_zeta = (nemitt_zeta / particle_ref._xobject.beta0[0]
+                        / particle_ref._xobject.gamma0[0])
 
 
         n_constraints = sum([vv is not None for vv in [x, x_norm, px, px_norm,
@@ -397,7 +418,7 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
     part_dict['zeta'] = XX[4, :]
     part_dict['ptau'] = XX[5, :] * particle_ref._xobject.beta0[0]
 
-    part_dict['weight'] = np.zeros(num_particles, dtype=np.int64)
+    part_dict['weight'] = np.ones(num_particles, dtype=np.float64)
 
     if _context is None and _buffer is None and line is not None:
         _context = line._buffer.context
@@ -428,4 +449,14 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
 
         particles.start_tracking_at_element = at_element
 
+    particles.spin_x[:num_particles] = particle_ref._xobject.spin_x[0]
+    particles.spin_y[:num_particles] = particle_ref._xobject.spin_y[0]
+    particles.spin_z[:num_particles] = particle_ref._xobject.spin_z[0]
+    if 'spin_x' in kwargs:
+        particles.spin_x[:num_particles] = kwargs['spin_x']
+    if 'spin_y' in kwargs:
+        particles.spin_y[:num_particles] = kwargs['spin_y']
+    if 'spin_z' in kwargs:
+        particles.spin_z[:num_particles] = kwargs['spin_z']
+
     return particles
