@@ -40,10 +40,14 @@ def _characterize_line(line, particle_ref,
         if ee.__class__.__name__ == 'Cavity':
             eecp = ee.copy(_context=xo.ContextCpu())
             if ee.voltage != 0:
-                freq_list.append(eecp.frequency)
                 lag_list_deg.append(eecp.lag)
                 voltage_list.append(eecp.voltage)
-                h_list.append(eecp.frequency*T_rev)
+                if eecp.frequency == 0:
+                    h_list.append(eecp.harmonic)
+                    freq_list.append(eecp.harmonic / T_rev)
+                else:
+                    h_list.append(eecp.frequency*T_rev)
+                    freq_list.append(eecp.frequency)
                 found_nonlinear_longitudinal = True
         elif ee.__class__.__name__ == 'LineSegmentMap':
             eecp = ee.copy(_context=xo.ContextCpu())
